﻿using Microsoft.AspNetCore.Http;
using MoYu.FriendlyException;
using SafeCampus.Core.Utils;
using SafeCampus.Core.Utils.Machine;
using SharpCompress.Common;
namespace SafeCampus.Web.Core;

public class SystemAuthorizeMiddleware
{
    private readonly RequestDelegate _next;
    private readonly IMachineUtil _machineUtil;

    public SystemAuthorizeMiddleware(RequestDelegate next, IMachineUtil machineUtil)
    {
        _next = next;
        _machineUtil = machineUtil;
    }
    // 排除不需要授权验证的接口
    private static readonly string[] ExcludedPaths = new[]
    {
        "/sys/auth/activate/activation",
        "/sys/auth/activate/machinecode",
    };
    public async Task InvokeAsync(HttpContext context)
    {
        var path = context.Request.Path.ToString().ToLower();
        // 排除不需要授权验证的接口
        if (ExcludedPaths.Contains(path))
        {
            await _next(context);
            return;
        }
        var filePath = Path.Combine(App.HostEnvironment.ContentRootPath, "activation.config");
        var msg = "";
        if (File.Exists(filePath))
        {
            var activationCode = await File.ReadAllTextAsync(filePath);
            var yuanwen1 = CryptogramUtil.Sm4Decrypt(activationCode);
            var yuanwen2 = CryptogramUtil.Sm2Decrypt(yuanwen1);
            if (!string.IsNullOrEmpty(yuanwen2))
            {
                var chaifen = yuanwen2.Split('$');
                var isCorrect = DateTime.TryParse(chaifen[3], out var activeTime);
                if (chaifen.Length == 4 && isCorrect && chaifen[0] == await _machineUtil.GetProcessorId() && chaifen[1] == await _machineUtil.GetDiskId() &&
                    chaifen[2] == await _machineUtil.GetMacAddress() && activeTime > DateTime.Now)
                {
                    // 调用下一个中间件
                    await _next(context);
                    return;
                }
            }
            msg = "授权已过期，请联系管理员授权";
        }
        else
            msg = "请联系管理员授权";
        var data = new { message = msg }; // 创建一个数据对象
        var json = JsonConvert.SerializeObject(data);
        context.Response.StatusCode = StatusCodes.Status421MisdirectedRequest;
        context.Response.ContentType = "application/json";
        await context.Response.WriteAsync(json);
    }
}